@@ -85,7 +85,7 @@ PRIVATE CLASS *Class;
 PRIVATE int NSection;
 #endif
 
-#define RELOCATE(_ptr) ((char *)(_ptr) = (char *)&class->string[(long)(_ptr)])
+#define RELOCATE(_ptr) (_ptr = (char *)&class->string[(long)(_ptr)])
 
 
 PRIVATE char *get_section(char *sec_name, char **section, long one, short *psize, char *swapcode)
@@ -294,9 +294,9 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
 
   if (_swap)
   {
-    SWAP_long(&header->magic);
-    SWAP_long(&header->version);
-    SWAP_long(&header->flag);
+    SWAP_long((long *)&header->magic);
+    SWAP_long((long *)&header->version);
+    SWAP_long((long *)&header->flag);
   }
 
   if (header->magic != OUTPUT_MAGIC)
