@@ -669,6 +669,7 @@ PUBLIC void EXEC_native(void)
   bool error;
   void *free;
   int n;
+  VALUE ret;
 
   /* vrification des paramtres */
 
@@ -755,6 +756,7 @@ PUBLIC void EXEC_native(void)
   }
 
   error = EXEC_call_native(desc->exec, object, desc->type, &SP[-nparam]);
+  ret = TEMP;
 
   /* Libration des arguments */
 
@@ -793,14 +795,14 @@ PUBLIC void EXEC_native(void)
     }
     else
     {
-      BORROW(&TEMP);
+      BORROW(&ret);
 
       if (drop)
-        RELEASE(&TEMP);
+        RELEASE(&ret);
       else
       {
-        VALUE_conv(&TEMP, desc->type);
-        *SP = TEMP;
+        VALUE_conv(&ret, desc->type);
+        *SP = ret;
         SP++;
       }
     }
