@@ -288,12 +288,14 @@ PUBLIC void TRACE_check_signal(void)
 PUBLIC void TRACE_exit(void)
 {
   ARRAY_delete(&Breakpoint);
-  
+
+  /*  
   if (EXEC_fifo)
   {
     fclose(_in);
     fclose(_out);
   }
+  */
 }
 
 
@@ -317,7 +319,7 @@ PRIVATE int find_free_breakpoint(void)
 
 PRIVATE boolean set_breakpoint(CLASS *class, ushort line)
 {
-  PCODE *addr;
+  PCODE *addr = NULL;
   TRACE_BREAK *brk;
   int id;
   FUNCTION *func;
