@@ -220,7 +220,7 @@ static void conv_data(char *data, GB_VARIANT_VALUE *val, Oid type)
 
           sscanf(data, "%2hu:%2hu:%lf", &date.hour, &date.min, &sec);
           date.sec = (short)sec;
-          date.msec = (short)(sec * 1000 + 0.5);
+          date.msec = (short)((sec - date.sec) * 1000 + 0.5);
           break;
 
         case TIMESTAMPOID:
@@ -233,7 +233,7 @@ static void conv_data(char *data, GB_VARIANT_VALUE *val, Oid type)
 
           sscanf(data, "%4hu-%2hu-%2hu %2hu:%2hu:%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
           date.sec = (short)sec;
-          date.msec = (short)(sec * 1000 + 0.5);
+          date.msec = (short)((sec - date.sec) * 1000 + 0.5);
           break;
       }
 
@@ -470,9 +470,7 @@ static void close_database(DB_DATABASE handle)
 
 static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add)
 {
-  char *s;
   int l;
-  int i;
   GB_DATE_SERIAL *date;
   bool bc;
 
@@ -490,22 +488,7 @@ static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add)
     case GB_T_STRING:
     case GB_T_CSTRING:
 
-      s = VALUE((GB_STRING *)arg).addr + VALUE((GB_STRING *)arg).start;
-      l = VALUE((GB_STRING *)arg).len;
-
-      if (arg->type == GB_T_STRING)
-         add("'", 1);
-
-      for (i = 0; i < l; i++, s++)
-      {
-        add(s, 1);
-        if (*s == '\'' || *s == '\\')
-          add(s, 1);
-      }
-      if (arg->type == GB_T_STRING)
-         add("'", 1);
-
-      return TRUE;
+      return FALSE;
 
     case GB_T_DATE:
 
@@ -680,7 +663,8 @@ static int field_index(DB_RESULT Result, char *name, DB_DATABASE handle, long ve
   char *table;
   int numfields, oid;
   char *qfield =
-    "select oid from pg_class where relname = lower('&1')";
+    "select oid from pg_class where relname = lower('&1') "
+    "and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
   fld = strchr(name, (int)FLD_SEP);
   
@@ -881,6 +865,7 @@ static int table_init(DB_DATABASE db, char *table, DB_INFO *info)
     "select pg_attribute.attname, pg_attribute.atttypid::int,pg_attribute.atttypmod "
       "from pg_class, pg_attribute "
       "where pg_class.relname = lower('&1') "
+        "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
         "and pg_attribute.attnum > 0 "
         "and pg_attribute.attrelid = pg_class.oid ";
 
@@ -957,6 +942,7 @@ static int table_index(DB_DATABASE db, char *table, DB_INFO *info)
     "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
       "from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
         "where pg_cl.relname = lower('&1_pkey') AND pg_cl.oid = pg_ind.indexrelid "
+          "and (pg_cl.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
           "and pg_att2.attrelid = pg_ind.indexrelid "
           "and pg_att1.attrelid = pg_ind.indrelid "
           "and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
@@ -1031,18 +1017,40 @@ static void table_release(DB_DATABASE db, DB_INFO *info)
 
 *****************************************************************************/
 
-static int table_exist(DB_DATABASE handle, char *table)
+static int table_exist(DB_DATABASE handle, char *table, long version)
 {
-  const char *query =
+  const char *query73 =
     "select relname from pg_class where (relkind = 'r' or relkind = 'v') "
-    "and (relname = lower('&1'))";
+    "and (relname = lower('&1')) "
+    "and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
+    
+  const char *query74_1 =
+    "select tablename from pg_tables where schemaname <> 'information_schema' and tablename = lower('&1')";
+
+  const char *query74_2 =
+    "select viewname from pg_views where schemaname <> 'information_schema' and viewname = lower('&1')";
 
   PGconn *conn = (PGconn *)handle;
   PGresult *res;
   int exist;
 
-  if (do_query(conn, "Unable to check table: &1", &res, query, 1, table))
-    return TRUE;
+  if (1) //version < 70400)
+  {
+    if (do_query(conn, "Unable to check table: &1", &res, query73, 1, table))
+      return TRUE;
+  }
+  else
+  {
+    if (do_query(conn, "Unable to check table: &1", &res, query74_1, 1, table))
+      return TRUE;
+      
+    if (PQntuples(res) == 0)
+    {
+      PQclear(res);
+      if (do_query(conn, "Unable to check table: &1", &res, query74_2, 1, table))    
+        return TRUE;
+    }
+  }
 
   exist = PQntuples(res) == 1;
 
@@ -1069,27 +1077,17 @@ static int table_exist(DB_DATABASE handle, char *table)
 
 *****************************************************************************/
 
-static long table_list(DB_DATABASE handle, char ***tables, long version)
+static long table_list_73(DB_DATABASE handle, char ***tables)
 {
-  /* depending on version */
-  const char *query73 =
-    "select relname from pg_class where (relkind = 'r' or relkind = 'v')";
-
-  const char *query74 =
-    "select tablename from pg_tables"; //7.4.1
- 
-  const char *query;
+  const char *query =
+    "select relname from pg_class where (relkind = 'r' or relkind = 'v') "
+    "and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
   PGconn *conn = (PGconn *)handle;
   PGresult *res;
   int i;
   long count;
 
-  if (version < 70400) // version 7.3.? 
-	  query = query73;
-  else
-	  query = query74;
-
   if (do_query(conn, "Unable to get tables: &1", &res, query, 0))
     return -1;
 
@@ -1108,6 +1106,55 @@ static long table_list(DB_DATABASE handle, char ***tables, long version)
   return count;
 }
 
+static long table_list_74(DB_DATABASE handle, char ***tables)
+{
+  const char *query1 =
+    "select tablename from pg_tables where schemaname <> 'information_schema'";
+
+  const char *query2 =
+    "select viewname from pg_views where schemaname <> 'information_schema'";
+
+  PGconn *conn = (PGconn *)handle;
+  PGresult *res1, *res2;
+  int i, j;
+  long count;
+
+  if (do_query(conn, "Unable to get tables: &1", &res1, query1, 0))
+    return -1;
+
+  if (do_query(conn, "Unable to get tables: &1", &res2, query2, 0))
+    return -1;
+
+  count = PQntuples(res1) + PQntuples(res2);
+
+  if (tables)
+  {
+    GB.NewArray(tables, sizeof(char *), count);
+
+    j = 0;
+    
+    for (i = 0; i < PQntuples(res1); i++, j++)
+      GB.NewString(&((*tables)[j]), PQgetvalue(res1, i, 0), 0);
+    
+    for (i = 0; i < PQntuples(res2); i++, j++)
+      GB.NewString(&((*tables)[j]), PQgetvalue(res2, i, 0), 0);
+  }
+
+  PQclear(res1);
+  PQclear(res2);
+
+  return count;
+}
+
+static long table_list(DB_DATABASE handle, char ***tables, long version)
+{
+  //if (version < 70400)
+    return table_list_73(handle, tables);
+  //else
+  //  return table_list_74(handle, tables);
+}
+
+
 
 /*****************************************************************************
 
@@ -1130,6 +1177,7 @@ static int table_primary_key(DB_DATABASE handle, char *table, char ***primary)
     "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
       "from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
         "where pg_cl.relname = lower('&1_pkey') AND pg_cl.oid = pg_ind.indexrelid "
+          "and (pg_cl.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
           "and pg_att2.attrelid = pg_ind.indexrelid "
           "and pg_att1.attrelid = pg_ind.indrelid "
           "and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
@@ -1170,32 +1218,47 @@ static int table_primary_key(DB_DATABASE handle, char *table, char ***primary)
 static int table_is_system(DB_DATABASE handle, char *table, long version)
 {
   const char *query =
+    "select 1 from pg_class where (relkind = 'r' or relkind = 'v') "
+    "and (relname = lower('&1')) "
+    "and (relnamespace not in (select oid from pg_namespace where nspname <> 'public'))";
+  
+  const char *query1 =
     "select 1 from pg_tables where tablename = lower('&1') and schemaname = 'public'";
 
+  const char *query2 =
+    "select 1 from pg_views where viewname = lower('&1') and schemaname = 'public'";
+
   PGconn *conn = (PGconn *)handle;
   PGresult *res;
   int exist;
 
   /* check version */
-  if (version < 70400){ // version 7.3.? 
+  /*if (version < 70400){ // version 7.3.? 
      if (table[0] == 'p' && table[1] == 'g' && table[2] == '_')
         return TRUE;
      else
         return FALSE;
-  }
+  }*/
     
   if (do_query(conn, "Unable to check table: &1", &res, query, 1, table))
     return FALSE;
 
   exist = PQntuples(res) == 1;
+  PQclear(res);
+  
+  if (exist)
+    return FALSE;
+  
+  if (do_query(conn, "Unable to check table: &1", &res, query2, 1, table))
+    return FALSE;
 
+  exist = PQntuples(res) == 1;
   PQclear(res);
 
   if (exist)
-     return FALSE;
-  else
-     return TRUE;
-
+    return FALSE;
+    
+  return TRUE;
 }
 
 
@@ -1337,6 +1400,7 @@ static int field_exist(DB_DATABASE handle, char *table, char *field)
 {
   const char *query = "select pg_attribute.attname from pg_class, pg_attribute "
     "where pg_class.relname = lower('&1') "
+    "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_attribute.attname = lower('&2') "
     "and pg_attribute.attnum > 0 "
     "and pg_attribute.attrelid = pg_class.oid ";
@@ -1376,6 +1440,7 @@ static long field_list(DB_DATABASE handle, char *table, char ***fields)
 {
   const char *query = "select pg_attribute.attname from pg_class, pg_attribute "
     "where pg_class.relname = lower('&1') "
+    "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_attribute.attnum > 0 "
     "and pg_attribute.attrelid = pg_class.oid";
 
@@ -1425,6 +1490,7 @@ static int field_info(DB_DATABASE handle, char *table, char *field, DB_FIELD *in
     "from pg_class, pg_attribute "
     "left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
     "where pg_class.relname = lower('&1') "
+    "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_attribute.attname = lower('&2') "
     "and pg_attribute.attnum > 0 "
     "and pg_attribute.attrelid = pg_class.oid";
@@ -1507,6 +1573,7 @@ static int index_exist(DB_DATABASE handle, char *table, char *index)
   const char *query =
     "select pg_class.relname from pg_class, pg_index, pg_class pg_class2 "
     "where pg_class2.relname = lower('&1') "
+    "and (pg_class2.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_index.indrelid = pg_class2.oid "
     "and pg_index.indexrelid = pg_class.oid "
     "and pg_class.relname = lower('&2')";
@@ -1550,6 +1617,7 @@ static long index_list(DB_DATABASE handle, char *table, char ***indexes)
 {
   const char *query = "select pg_class.relname from pg_class, pg_index, pg_class pg_class2 "
     "where pg_class2.relname = lower('&1') "
+    "and (pg_class2.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_index.indrelid = pg_class2.oid "
     "and pg_index.indexrelid = pg_class.oid ";
 
@@ -1596,6 +1664,7 @@ static int index_info(DB_DATABASE handle, char *table, char *index, DB_INDEX *in
   const char *query =
     "select indisunique, indisprimary, indexrelid from pg_class, pg_index, pg_class pg_class2 "
     "where pg_class2.relname = lower('&1') "
+    "and (pg_class2.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_index.indrelid = pg_class2.oid "
     "and pg_index.indexrelid = pg_class.oid "
     "and pg_class.relname = lower('&2')";
