@@ -512,6 +512,9 @@ BEGIN_METHOD_VOID(CWINDOW_show)
     //if (CWINDOW_Current)
     //  GB.Error("A modal window is already displayed");
     //else
+    if (CWINDOW_Current)
+      WINDOW->showModal();
+    else
       WINDOW->showActivate();
   }
 
@@ -1036,6 +1039,7 @@ MyMainWindow::MyMainWindow(QWidget *parent) :
   border = BorderResizable;
   state = StateNormal;
   mustCenter = false;
+  _activate = false;
 
   mdichild = parent && parent->isA("QWorkspace");
   if (mdichild)
@@ -1222,6 +1226,16 @@ void MyMainWindow::setState(int s)
 }
 
 
+void MyMainWindow::showEvent(QShowEvent *e)
+{
+  if (_activate)
+  {
+    raise();
+    setActiveWindow();
+    _activate = false;
+  }
+}
+
 void MyMainWindow::showActivate()
 {
   CWIDGET *_object = CWidget::get(this);
@@ -1272,7 +1286,10 @@ void MyMainWindow::showActivate()
   else
   {
     if (st == StateMinimized)
+    {
       setState(StateNormal);
+      _activate = true;
+    }
     else if (!isVisible())
     {
       //qDebug("#1");
@@ -1301,8 +1318,6 @@ void MyMainWindow::showActivate()
 
   }
 
-  //qApp->syncX();
-  
   defineMask();
   
   CWIDGET_clear_flag(THIS, WF_IN_SHOW);
